==> pulsar-client-all-3.3.5-sources.jar: org/apache/pulsar/common/api/proto/CommandEndTxnOnPartition.java <==
package org.apache.pulsar.common.api.proto;
public final class CommandEndTxnOnPartition {
	private long requestId;
	private static final int _REQUEST_ID_FIELD_NUMBER = 1;
	private static final int _REQUEST_ID_TAG = (_REQUEST_ID_FIELD_NUMBER << LightProtoCodec.TAG_TYPE_BITS)
			| LightProtoCodec.WIRETYPE_VARINT;
	private static final int _REQUEST_ID_TAG_SIZE = LightProtoCodec.computeVarIntSize(_REQUEST_ID_TAG);
	private static final int _REQUEST_ID_MASK = 1 << (0 % 32);
	public boolean hasRequestId() {
		return (_bitField0 & _REQUEST_ID_MASK) != 0;
	}
	public long getRequestId() {
		if (!hasRequestId()) {
			throw new IllegalStateException("Field 'request_id' is not set");
		}
		return requestId;
	}
	public CommandEndTxnOnPartition setRequestId(long requestId) {
		this.requestId = requestId;
		_bitField0 |= _REQUEST_ID_MASK;
		_cachedSize = -1;
		return this;
	}
	public CommandEndTxnOnPartition clearRequestId() {
		_bitField0 &= ~_REQUEST_ID_MASK;
		return this;
	}

	private long txnidLeastBits = 0;
	private static final int _TXNID_LEAST_BITS_FIELD_NUMBER = 2;
	private static final int _TXNID_LEAST_BITS_TAG = (_TXNID_LEAST_BITS_FIELD_NUMBER << LightProtoCodec.TAG_TYPE_BITS)
			| LightProtoCodec.WIRETYPE_VARINT;
	private static final int _TXNID_LEAST_BITS_TAG_SIZE = LightProtoCodec.computeVarIntSize(_TXNID_LEAST_BITS_TAG);
	private static final int _TXNID_LEAST_BITS_MASK = 1 << (1 % 32);
	public boolean hasTxnidLeastBits() {
		return (_bitField0 & _TXNID_LEAST_BITS_MASK) != 0;
	}
	public long getTxnidLeastBits() {
		return txnidLeastBits;
	}
	public CommandEndTxnOnPartition setTxnidLeastBits(long txnidLeastBits) {
		this.txnidLeastBits = txnidLeastBits;
		_bitField0 |= _TXNID_LEAST_BITS_MASK;
		_cachedSize = -1;
		return this;
	}
	public CommandEndTxnOnPartition clearTxnidLeastBits() {
		_bitField0 &= ~_TXNID_LEAST_BITS_MASK;
		txnidLeastBits = 0;
		return this;
	}

	private long txnidMostBits = 0;
	private static final int _TXNID_MOST_BITS_FIELD_NUMBER = 3;
	private static final int _TXNID_MOST_BITS_TAG = (_TXNID_MOST_BITS_FIELD_NUMBER << LightProtoCodec.TAG_TYPE_BITS)
			| LightProtoCodec.WIRETYPE_VARINT;
	private static final int _TXNID_MOST_BITS_TAG_SIZE = LightProtoCodec.computeVarIntSize(_TXNID_MOST_BITS_TAG);
	private static final int _TXNID_MOST_BITS_MASK = 1 << (2 % 32);
	public boolean hasTxnidMostBits() {
		return (_bitField0 & _TXNID_MOST_BITS_MASK) != 0;
	}
	public long getTxnidMostBits() {
		return txnidMostBits;
	}
	public CommandEndTxnOnPartition setTxnidMostBits(long txnidMostBits) {
		this.txnidMostBits = txnidMostBits;
		_bitField0 |= _TXNID_MOST_BITS_MASK;
		_cachedSize = -1;
		return this;
	}
	public CommandEndTxnOnPartition clearTxnidMostBits() {
		_bitField0 &= ~_TXNID_MOST_BITS_MASK;
		txnidMostBits = 0;
		return this;
	}

	private String topic;
	private int _topicBufferIdx = -1;
	private int _topicBufferLen = -1;
	private static final int _TOPIC_FIELD_NUMBER = 4;
	private static final int _TOPIC_TAG = (_TOPIC_FIELD_NUMBER << LightProtoCodec.TAG_TYPE_BITS)
			| LightProtoCodec.WIRETYPE_LENGTH_DELIMITED;
	private static final int _TOPIC_TAG_SIZE = LightProtoCodec.computeVarIntSize(_TOPIC_TAG);
	private static final int _TOPIC_MASK = 1 << (3 % 32);
	public boolean hasTopic() {
		return (_bitField0 & _TOPIC_MASK) != 0;
	}
	public String getTopic() {
		if (!hasTopic()) {
			throw new IllegalStateException("Field 'topic' is not set");
		}
		if (topic == null) {
			topic = LightProtoCodec.readString(_parsedBuffer, _topicBufferIdx, _topicBufferLen);
		}
		return topic;
	}
	public CommandEndTxnOnPartition setTopic(String topic) {
		this.topic = topic;
		_bitField0 |= _TOPIC_MASK;
		_topicBufferIdx = -1;
		_topicBufferLen = LightProtoCodec.computeStringUTF8Size(topic);
		_cachedSize = -1;
		return this;
	}
	public CommandEndTxnOnPartition clearTopic() {
		_bitField0 &= ~_TOPIC_MASK;
		topic = null;
		_topicBufferIdx = -1;
		_topicBufferLen = -1;
		return this;
	}

	private TxnAction txnAction;
	private static final int _TXN_ACTION_FIELD_NUMBER = 5;
	private static final int _TXN_ACTION_TAG = (_TXN_ACTION_FIELD_NUMBER << LightProtoCodec.TAG_TYPE_BITS)
			| LightProtoCodec.WIRETYPE_VARINT;
	private static final int _TXN_ACTION_TAG_SIZE = LightProtoCodec.computeVarIntSize(_TXN_ACTION_TAG);
	private static final int _TXN_ACTION_MASK = 1 << (4 % 32);
	public boolean hasTxnAction() {
		return (_bitField0 & _TXN_ACTION_MASK) != 0;
	}
	public TxnAction getTxnAction() {
		if (!hasTxnAction()) {
			throw new IllegalStateException("Field 'txn_action' is not set");
		}
		return txnAction;
	}
	public CommandEndTxnOnPartition setTxnAction(TxnAction txnAction) {
		this.txnAction = txnAction;
		_bitField0 |= _TXN_ACTION_MASK;
		_cachedSize = -1;
		return this;
	}
	public CommandEndTxnOnPartition clearTxnAction() {
		_bitField0 &= ~_TXN_ACTION_MASK;
		return this;
	}

	private long txnidLeastBitsOfLowWatermark;
	private static final int _TXNID_LEAST_BITS_OF_LOW_WATERMARK_FIELD_NUMBER = 6;
	private static final int _TXNID_LEAST_BITS_OF_LOW_WATERMARK_TAG = (_TXNID_LEAST_BITS_OF_LOW_WATERMARK_FIELD_NUMBER << LightProtoCodec.TAG_TYPE_BITS)
			| LightProtoCodec.WIRETYPE_VARINT;
	private static final int _TXNID_LEAST_BITS_OF_LOW_WATERMARK_TAG_SIZE = LightProtoCodec
			.computeVarIntSize(_TXNID_LEAST_BITS_OF_LOW_WATERMARK_TAG);
	private static final int _TXNID_LEAST_BITS_OF_LOW_WATERMARK_MASK = 1 << (5 % 32);
	public boolean hasTxnidLeastBitsOfLowWatermark() {
		return (_bitField0 & _TXNID_LEAST_BITS_OF_LOW_WATERMARK_MASK) != 0;
	}
	public long getTxnidLeastBitsOfLowWatermark() {
		if (!hasTxnidLeastBitsOfLowWatermark()) {
			throw new IllegalStateException("Field 'txnid_least_bits_of_low_watermark' is not set");
		}
		return txnidLeastBitsOfLowWatermark;
	}
	public CommandEndTxnOnPartition setTxnidLeastBitsOfLowWatermark(long txnidLeastBitsOfLowWatermark) {
		this.txnidLeastBitsOfLowWatermark = txnidLeastBitsOfLowWatermark;
		_bitField0 |= _TXNID_LEAST_BITS_OF_LOW_WATERMARK_MASK;
		_cachedSize = -1;
		return this;
	}
	public CommandEndTxnOnPartition clearTxnidLeastBitsOfLowWatermark() {
		_bitField0 &= ~_TXNID_LEAST_BITS_OF_LOW_WATERMARK_MASK;
		return this;
	}

	private int _bitField0;
	private static final int _REQUIRED_FIELDS_MASK0 = 0 | _REQUEST_ID_MASK;
	public int writeTo(org.apache.pulsar.shade.io.netty.buffer.ByteBuf _b) {
		checkRequiredFields();
		int _writeIdx = _b.writerIndex();
		LightProtoCodec.writeVarInt(_b, _REQUEST_ID_TAG);
		LightProtoCodec.writeVarInt64(_b, requestId);
		if (hasTxnidLeastBits()) {
			LightProtoCodec.writeVarInt(_b, _TXNID_LEAST_BITS_TAG);
			LightProtoCodec.writeVarInt64(_b, txnidLeastBits);
		}
		if (hasTxnidMostBits()) {
			LightProtoCodec.writeVarInt(_b, _TXNID_MOST_BITS_TAG);
			LightProtoCodec.writeVarInt64(_b, txnidMostBits);
		}
		if (hasTopic()) {
			LightProtoCodec.writeVarInt(_b, _TOPIC_TAG);
			LightProtoCodec.writeVarInt(_b, _topicBufferLen);
			if (_topicBufferIdx == -1) {
				LightProtoCodec.writeString(_b, topic, _topicBufferLen);
			} else {
				_parsedBuffer.getBytes(_topicBufferIdx, _b, _topicBufferLen);
			}
		}
		if (hasTxnAction()) {
			LightProtoCodec.writeVarInt(_b, _TXN_ACTION_TAG);
			LightProtoCodec.writeVarInt(_b, txnAction.getValue());
		}
		if (hasTxnidLeastBitsOfLowWatermark()) {
			LightProtoCodec.writeVarInt(_b, _TXNID_LEAST_BITS_OF_LOW_WATERMARK_TAG);
			LightProtoCodec.writeVarInt64(_b, txnidLeastBitsOfLowWatermark);
		}
		return (_b.writerIndex() - _writeIdx);
	}
	public int getSerializedSize() {
		if (_cachedSize > -1) {
			return _cachedSize;
		}

		int _size = 0;
		_size += _REQUEST_ID_TAG_SIZE;
		_size += LightProtoCodec.computeVarInt64Size(requestId);
		if (hasTxnidLeastBits()) {
			_size += _TXNID_LEAST_BITS_TAG_SIZE;
			_size += LightProtoCodec.computeVarInt64Size(txnidLeastBits);
		}
		if (hasTxnidMostBits()) {
			_size += _TXNID_MOST_BITS_TAG_SIZE;
			_size += LightProtoCodec.computeVarInt64Size(txnidMostBits);
		}
		if (hasTopic()) {
			_size += _TOPIC_TAG_SIZE;
			_size += LightProtoCodec.computeVarIntSize(_topicBufferLen);
			_size += _topicBufferLen;
		}
		if (hasTxnAction()) {
			_size += _TXN_ACTION_TAG_SIZE;
			_size += LightProtoCodec.computeVarIntSize(txnAction.getValue());
		}
		if (hasTxnidLeastBitsOfLowWatermark()) {
			_size += _TXNID_LEAST_BITS_OF_LOW_WATERMARK_TAG_SIZE;
			_size += LightProtoCodec.computeVarInt64Size(txnidLeastBitsOfLowWatermark);
		}
		_cachedSize = _size;
		return _size;
	}
	public void parseFrom(org.apache.pulsar.shade.io.netty.buffer.ByteBuf _buffer, int _size) {
		clear();
		int _endIdx = _buffer.readerIndex() + _size;
		while (_buffer.readerIndex() < _endIdx) {
			int _tag = LightProtoCodec.readVarInt(_buffer);
			switch (_tag) {
				case _REQUEST_ID_TAG :
					_bitField0 |= _REQUEST_ID_MASK;
					requestId = LightProtoCodec.readVarInt64(_buffer);
					break;
				case _TXNID_LEAST_BITS_TAG :
					_bitField0 |= _TXNID_LEAST_BITS_MASK;
					txnidLeastBits = LightProtoCodec.readVarInt64(_buffer);
					break;
				case _TXNID_MOST_BITS_TAG :
					_bitField0 |= _TXNID_MOST_BITS_MASK;
					txnidMostBits = LightProtoCodec.readVarInt64(_buffer);
					break;
				case _TOPIC_TAG :
					_bitField0 |= _TOPIC_MASK;
					_topicBufferLen = LightProtoCodec.readVarInt(_buffer);
					_topicBufferIdx = _buffer.readerIndex();
					_buffer.skipBytes(_topicBufferLen);
					break;
				case _TXN_ACTION_TAG :
					TxnAction _txnAction = TxnAction.valueOf(LightProtoCodec.readVarInt(_buffer));
					if (_txnAction != null) {
						_bitField0 |= _TXN_ACTION_MASK;
						txnAction = _txnAction;
					}
					break;
				case _TXNID_LEAST_BITS_OF_LOW_WATERMARK_TAG :
					_bitField0 |= _TXNID_LEAST_BITS_OF_LOW_WATERMARK_MASK;
					txnidLeastBitsOfLowWatermark = LightProtoCodec.readVarInt64(_buffer);
					break;
				default :
					LightProtoCodec.skipUnknownField(_tag, _buffer);
			}
		}
		checkRequiredFields();
		_parsedBuffer = _buffer;
	}
	private void checkRequiredFields() {
		if ((_bitField0 & _REQUIRED_FIELDS_MASK0) != _REQUIRED_FIELDS_MASK0) {
			throw new IllegalStateException("Some required fields are missing");
		}
	}
	public CommandEndTxnOnPartition clear() {
		txnidLeastBits = 0;
		txnidMostBits = 0;
		topic = null;
		_topicBufferIdx = -1;
		_topicBufferLen = -1;
		_parsedBuffer = null;
		_cachedSize = -1;
		_bitField0 = 0;
		return this;
	}
	public CommandEndTxnOnPartition copyFrom(CommandEndTxnOnPartition _other) {
		_cachedSize = -1;
		if (_other.hasRequestId()) {
			setRequestId(_other.requestId);
		}
		if (_other.hasTxnidLeastBits()) {
			setTxnidLeastBits(_other.txnidLeastBits);
		}
		if (_other.hasTxnidMostBits()) {
			setTxnidMostBits(_other.txnidMostBits);
		}
		if (_other.hasTopic()) {
			setTopic(_other.getTopic());
		}
		if (_other.hasTxnAction()) {
			setTxnAction(_other.txnAction);
		}
		if (_other.hasTxnidLeastBitsOfLowWatermark()) {
			setTxnidLeastBitsOfLowWatermark(_other.txnidLeastBitsOfLowWatermark);
		}
		return this;
	}
	public byte[] toByteArray() {
		byte[] a = new byte[getSerializedSize()];
		io.netty.buffer.ByteBuf b = io.netty.buffer.Unpooled.wrappedBuffer(a).writerIndex(0);
		this.writeTo(b);
		return a;
	}
	public void parseFrom(byte[] a) {
		io.netty.buffer.ByteBuf b = io.netty.buffer.Unpooled.wrappedBuffer(a);
		this.parseFrom(b, b.readableBytes());
	}
	private int _cachedSize;

	private org.apache.pulsar.shade.io.netty.buffer.ByteBuf _parsedBuffer;

}

==> pulsar-client-all-4.0.5-sources.jar: org/apache/pulsar/common/api/proto/CommandEndTxnOnPartition.java <==
package org.apache.pulsar.common.api.proto;
public final class CommandEndTxnOnPartition {
	private long requestId;
	private static final int _REQUEST_ID_FIELD_NUMBER = 1;
	private static final int _REQUEST_ID_TAG = (_REQUEST_ID_FIELD_NUMBER << LightProtoCodec.TAG_TYPE_BITS)
			| LightProtoCodec.WIRETYPE_VARINT;
	private static final int _REQUEST_ID_TAG_SIZE = LightProtoCodec.computeVarIntSize(_REQUEST_ID_TAG);
	private static final int _REQUEST_ID_MASK = 1 << (0 % 32);
	public boolean hasRequestId() {
		return (_bitField0 & _REQUEST_ID_MASK) != 0;
	}
	public long getRequestId() {
		if (!hasRequestId()) {
			throw new IllegalStateException("Field 'request_id' is not set");
		}
		return requestId;
	}
	public CommandEndTxnOnPartition setRequestId(long requestId) {
		this.requestId = requestId;
		_bitField0 |= _REQUEST_ID_MASK;
		_cachedSize = -1;
		return this;
	}
	public CommandEndTxnOnPartition clearRequestId() {
		_bitField0 &= ~_REQUEST_ID_MASK;
		return this;
	}

	private long txnidLeastBits = 0;
	private static final int _TXNID_LEAST_BITS_FIELD_NUMBER = 2;
	private static final int _TXNID_LEAST_BITS_TAG = (_TXNID_LEAST_BITS_FIELD_NUMBER << LightProtoCodec.TAG_TYPE_BITS)
			| LightProtoCodec.WIRETYPE_VARINT;
	private static final int _TXNID_LEAST_BITS_TAG_SIZE = LightProtoCodec.computeVarIntSize(_TXNID_LEAST_BITS_TAG);
	private static final int _TXNID_LEAST_BITS_MASK = 1 << (1 % 32);
	public boolean hasTxnidLeastBits() {
		return (_bitField0 & _TXNID_LEAST_BITS_MASK) != 0;
	}
	public long getTxnidLeastBits() {
		return txnidLeastBits;
	}
	public CommandEndTxnOnPartition setTxnidLeastBits(long txnidLeastBits) {
		this.txnidLeastBits = txnidLeastBits;
		_bitField0 |= _TXNID_LEAST_BITS_MASK;
		_cachedSize = -1;
		return this;
	}
	public CommandEndTxnOnPartition clearTxnidLeastBits() {
		_bitField0 &= ~_TXNID_LEAST_BITS_MASK;
		txnidLeastBits = 0;
		return this;
	}

	private long txnidMostBits = 0;
	private static final int _TXNID_MOST_BITS_FIELD_NUMBER = 3;
	private static final int _TXNID_MOST_BITS_TAG = (_TXNID_MOST_BITS_FIELD_NUMBER << LightProtoCodec.TAG_TYPE_BITS)
			| LightProtoCodec.WIRETYPE_VARINT;
	private static final int _TXNID_MOST_BITS_TAG_SIZE = LightProtoCodec.computeVarIntSize(_TXNID_MOST_BITS_TAG);
	private static final int _TXNID_MOST_BITS_MASK = 1 << (2 % 32);
	public boolean hasTxnidMostBits() {
		return (_bitField0 & _TXNID_MOST_BITS_MASK) != 0;
	}
	public long getTxnidMostBits() {
		return txnidMostBits;
	}
	public CommandEndTxnOnPartition setTxnidMostBits(long txnidMostBits) {
		this.txnidMostBits = txnidMostBits;
		_bitField0 |= _TXNID_MOST_BITS_MASK;
		_cachedSize = -1;
		return this;
	}
	public CommandEndTxnOnPartition clearTxnidMostBits() {
		_bitField0 &= ~_TXNID_MOST_BITS_MASK;
		txnidMostBits = 0;
		return this;
	}

	private String topic;
	private int _topicBufferIdx = -1;
	private int _topicBufferLen = -1;
	private static final int _TOPIC_FIELD_NUMBER = 4;
	private static final int _TOPIC_TAG = (_TOPIC_FIELD_NUMBER << LightProtoCodec.TAG_TYPE_BITS)
			| LightProtoCodec.WIRETYPE_LENGTH_DELIMITED;
	private static final int _TOPIC_TAG_SIZE = LightProtoCodec.computeVarIntSize(_TOPIC_TAG);
	private static final int _TOPIC_MASK = 1 << (3 % 32);
	public boolean hasTopic() {
		return (_bitField0 & _TOPIC_MASK) != 0;
	}
	public String getTopic() {
		if (!hasTopic()) {
			throw new IllegalStateException("Field 'topic' is not set");
		}
		if (topic == null) {
			topic = LightProtoCodec.readString(_parsedBuffer, _topicBufferIdx, _topicBufferLen);
		}
		return topic;
	}
	public CommandEndTxnOnPartition setTopic(String topic) {
		this.topic = topic;
		_bitField0 |= _TOPIC_MASK;
		_topicBufferIdx = -1;
		_topicBufferLen = LightProtoCodec.computeStringUTF8Size(topic);
		_cachedSize = -1;
		return this;
	}
	public CommandEndTxnOnPartition clearTopic() {
		_bitField0 &= ~_TOPIC_MASK;
		topic = null;
		_topicBufferIdx = -1;
		_topicBufferLen = -1;
		return this;
	}

	private TxnAction txnAction;
	private static final int _TXN_ACTION_FIELD_NUMBER = 5;
	private static final int _TXN_ACTION_TAG = (_TXN_ACTION_FIELD_NUMBER << LightProtoCodec.TAG_TYPE_BITS)
			| LightProtoCodec.WIRETYPE_VARINT;
	private static final int _TXN_ACTION_TAG_SIZE = LightProtoCodec.computeVarIntSize(_TXN_ACTION_TAG);
	private static final int _TXN_ACTION_MASK = 1 << (4 % 32);
	public boolean hasTxnAction() {
		return (_bitField0 & _TXN_ACTION_MASK) != 0;
	}
	public TxnAction getTxnAction() {
		if (!hasTxnAction()) {
			throw new IllegalStateException("Field 'txn_action' is not set");
		}
		return txnAction;
	}
	public CommandEndTxnOnPartition setTxnAction(TxnAction txnAction) {
		this.txnAction = txnAction;
		_bitField0 |= _TXN_ACTION_MASK;
		_cachedSize = -1;
		return this;
	}
	public CommandEndTxnOnPartition clearTxnAction() {
		_bitField0 &= ~_TXN_ACTION_MASK;
		return this;
	}

	private long txnidLeastBitsOfLowWatermark;
	private static final int _TXNID_LEAST_BITS_OF_LOW_WATERMARK_FIELD_NUMBER = 6;
	private static final int _TXNID_LEAST_BITS_OF_LOW_WATERMARK_TAG = (_TXNID_LEAST_BITS_OF_LOW_WATERMARK_FIELD_NUMBER << LightProtoCodec.TAG_TYPE_BITS)
			| LightProtoCodec.WIRETYPE_VARINT;
	private static final int _TXNID_LEAST_BITS_OF_LOW_WATERMARK_TAG_SIZE = LightProtoCodec
			.computeVarIntSize(_TXNID_LEAST_BITS_OF_LOW_WATERMARK_TAG);
	private static final int _TXNID_LEAST_BITS_OF_LOW_WATERMARK_MASK = 1 << (5 % 32);
	public boolean hasTxnidLeastBitsOfLowWatermark() {
		return (_bitField0 & _TXNID_LEAST_BITS_OF_LOW_WATERMARK_MASK) != 0;
	}
	public long getTxnidLeastBitsOfLowWatermark() {
		if (!hasTxnidLeastBitsOfLowWatermark()) {
			throw new IllegalStateException("Field 'txnid_least_bits_of_low_watermark' is not set");
		}
		return txnidLeastBitsOfLowWatermark;
	}
	public CommandEndTxnOnPartition setTxnidLeastBitsOfLowWatermark(long txnidLeastBitsOfLowWatermark) {
		this.txnidLeastBitsOfLowWatermark = txnidLeastBitsOfLowWatermark;
		_bitField0 |= _TXNID_LEAST_BITS_OF_LOW_WATERMARK_MASK;
		_cachedSize = -1;
		return this;
	}
	public CommandEndTxnOnPartition clearTxnidLeastBitsOfLowWatermark() {
		_bitField0 &= ~_TXNID_LEAST_BITS_OF_LOW_WATERMARK_MASK;
		return this;
	}

	private int _bitField0;
	private static final int _REQUIRED_FIELDS_MASK0 = 0 | _REQUEST_ID_MASK;
	public int writeTo(org.apache.pulsar.shade.io.netty.buffer.ByteBuf _b) {
		checkRequiredFields();
		int _writeIdx = _b.writerIndex();
		LightProtoCodec.writeVarInt(_b, _REQUEST_ID_TAG);
		LightProtoCodec.writeVarInt64(_b, requestId);
		if (hasTxnidLeastBits()) {
			LightProtoCodec.writeVarInt(_b, _TXNID_LEAST_BITS_TAG);
			LightProtoCodec.writeVarInt64(_b, txnidLeastBits);
		}
		if (hasTxnidMostBits()) {
			LightProtoCodec.writeVarInt(_b, _TXNID_MOST_BITS_TAG);
			LightProtoCodec.writeVarInt64(_b, txnidMostBits);
		}
		if (hasTopic()) {
			LightProtoCodec.writeVarInt(_b, _TOPIC_TAG);
			LightProtoCodec.writeVarInt(_b, _topicBufferLen);
			if (_topicBufferIdx == -1) {
				LightProtoCodec.writeString(_b, topic, _topicBufferLen);
			} else {
				_parsedBuffer.getBytes(_topicBufferIdx, _b, _topicBufferLen);
			}
		}
		if (hasTxnAction()) {
			LightProtoCodec.writeVarInt(_b, _TXN_ACTION_TAG);
			LightProtoCodec.writeVarInt(_b, txnAction.getValue());
		}
		if (hasTxnidLeastBitsOfLowWatermark()) {
			LightProtoCodec.writeVarInt(_b, _TXNID_LEAST_BITS_OF_LOW_WATERMARK_TAG);
			LightProtoCodec.writeVarInt64(_b, txnidLeastBitsOfLowWatermark);
		}
		return (_b.writerIndex() - _writeIdx);
	}
	public int getSerializedSize() {
		if (_cachedSize > -1) {
			return _cachedSize;
		}

		int _size = 0;
		_size += _REQUEST_ID_TAG_SIZE;
		_size += LightProtoCodec.computeVarInt64Size(requestId);
		if (hasTxnidLeastBits()) {
			_size += _TXNID_LEAST_BITS_TAG_SIZE;
			_size += LightProtoCodec.computeVarInt64Size(txnidLeastBits);
		}
		if (hasTxnidMostBits()) {
			_size += _TXNID_MOST_BITS_TAG_SIZE;
			_size += LightProtoCodec.computeVarInt64Size(txnidMostBits);
		}
		if (hasTopic()) {
			_size += _TOPIC_TAG_SIZE;
			_size += LightProtoCodec.computeVarIntSize(_topicBufferLen);
			_size += _topicBufferLen;
		}
		if (hasTxnAction()) {
			_size += _TXN_ACTION_TAG_SIZE;
			_size += LightProtoCodec.computeVarIntSize(txnAction.getValue());
		}
		if (hasTxnidLeastBitsOfLowWatermark()) {
			_size += _TXNID_LEAST_BITS_OF_LOW_WATERMARK_TAG_SIZE;
			_size += LightProtoCodec.computeVarInt64Size(txnidLeastBitsOfLowWatermark);
		}
		_cachedSize = _size;
		return _size;
	}
	public void parseFrom(org.apache.pulsar.shade.io.netty.buffer.ByteBuf _buffer, int _size) {
		clear();
		int _endIdx = _buffer.readerIndex() + _size;
		while (_buffer.readerIndex() < _endIdx) {
			int _tag = LightProtoCodec.readVarInt(_buffer);
			switch (_tag) {
				case _REQUEST_ID_TAG :
					_bitField0 |= _REQUEST_ID_MASK;
					requestId = LightProtoCodec.readVarInt64(_buffer);
					break;
				case _TXNID_LEAST_BITS_TAG :
					_bitField0 |= _TXNID_LEAST_BITS_MASK;
					txnidLeastBits = LightProtoCodec.readVarInt64(_buffer);
					break;
				case _TXNID_MOST_BITS_TAG :
					_bitField0 |= _TXNID_MOST_BITS_MASK;
					txnidMostBits = LightProtoCodec.readVarInt64(_buffer);
					break;
				case _TOPIC_TAG :
					_bitField0 |= _TOPIC_MASK;
					_topicBufferLen = LightProtoCodec.readVarInt(_buffer);
					_topicBufferIdx = _buffer.readerIndex();
					_buffer.skipBytes(_topicBufferLen);
					break;
				case _TXN_ACTION_TAG :
					TxnAction _txnAction = TxnAction.valueOf(LightProtoCodec.readVarInt(_buffer));
					if (_txnAction != null) {
						_bitField0 |= _TXN_ACTION_MASK;
						txnAction = _txnAction;
					}
					break;
				case _TXNID_LEAST_BITS_OF_LOW_WATERMARK_TAG :
					_bitField0 |= _TXNID_LEAST_BITS_OF_LOW_WATERMARK_MASK;
					txnidLeastBitsOfLowWatermark = LightProtoCodec.readVarInt64(_buffer);
					break;
				default :
					LightProtoCodec.skipUnknownField(_tag, _buffer);
			}
		}
		checkRequiredFields();
		_parsedBuffer = _buffer;
	}
	private void checkRequiredFields() {
		if ((_bitField0 & _REQUIRED_FIELDS_MASK0) != _REQUIRED_FIELDS_MASK0) {
			throw new IllegalStateException("Some required fields are missing");
		}
	}
	public CommandEndTxnOnPartition clear() {
		txnidLeastBits = 0;
		txnidMostBits = 0;
		topic = null;
		_topicBufferIdx = -1;
		_topicBufferLen = -1;
		_parsedBuffer = null;
		_cachedSize = -1;
		_bitField0 = 0;
		return this;
	}
	public CommandEndTxnOnPartition copyFrom(CommandEndTxnOnPartition _other) {
		_cachedSize = -1;
		if (_other.hasRequestId()) {
			setRequestId(_other.requestId);
		}
		if (_other.hasTxnidLeastBits()) {
			setTxnidLeastBits(_other.txnidLeastBits);
		}
		if (_other.hasTxnidMostBits()) {
			setTxnidMostBits(_other.txnidMostBits);
		}
		if (_other.hasTopic()) {
			setTopic(_other.getTopic());
		}
		if (_other.hasTxnAction()) {
			setTxnAction(_other.txnAction);
		}
		if (_other.hasTxnidLeastBitsOfLowWatermark()) {
			setTxnidLeastBitsOfLowWatermark(_other.txnidLeastBitsOfLowWatermark);
		}
		return this;
	}
	public byte[] toByteArray() {
		byte[] a = new byte[getSerializedSize()];
		org.apache.pulsar.shade.io.netty.buffer.ByteBuf b = org.apache.pulsar.shade.io.netty.buffer.Unpooled.wrappedBuffer(a).writerIndex(0);
		this.writeTo(b);
		return a;
	}
	public void parseFrom(byte[] a) {
		org.apache.pulsar.shade.io.netty.buffer.ByteBuf b = org.apache.pulsar.shade.io.netty.buffer.Unpooled.wrappedBuffer(a);
		this.parseFrom(b, b.readableBytes());
	}
	private int _cachedSize;

	private org.apache.pulsar.shade.io.netty.buffer.ByteBuf _parsedBuffer;

}
